/******************************************************************************
Album filter plugin for Avisynth by V.C.Mohan
On a base clip  resized image from of right clip is overlaid. The position of clips, 
preset or custom position at which to display are provided. The start frame, end frame,
 corresponding RightClip frame number and coordinates are parameters. If preset is true,
 x and y coordinates are disregarded. If preset is false, nphotos and position, border
 are disregarded. nphotos number does not include base clip. A simpler version
 Album simple have fewer options.
********************************************************************************/
/*  
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
*/
class Album : public GenericVideoFilter {
		PClip RightClip;	// Album clip. Clip from which a frame is resized as a Photo 
		bool preset;		// IAre positions presets or custom?
		int nphotos;		// number of Photos  in this Album page
		int position;		// Preset position number at which Photo appears
		int startframe;		// Base clip frame number at which Photo appears
		int endframe;		// Base clip frame number at which Photo disappears
		int photoframe;		// Album clip frame number to be used for photo
		
		int border;			// border pixels. not more than (bwd(and bht)/number of positions)/40
		int inxl, inyt, inxr, inyb;
		int outxl, outyt, outxr, outyb;
		int root;
		
		bool still;			// resized photo is one frame or its video clip all frames in range resized
		 
  public:
							//Definition of function
    Album(PClip _child, PClip _RightClip,bool _preset,
				int _nphotos, int _position, 
				int _startframe, int _endframe,
				int _photoframe,int _border,
				int _inxl, int _inyt,
				int _inxr, int _inyb,
				int _outxl, int _outyt,
				int _outxr, int _outyb,
				  
				IScriptEnvironment* env) ;	
			
 	~Album();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
//void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
//bool __stdcall GetParity(int n);

};	

//Here is the acutal constructor code used

Album::Album(PClip _child, PClip _RightClip,bool _preset,
					 int _nphotos, int _position, 
				int _startframe, int _endframe,
				int _photoframe,int _border,
				int _inxl, int _inyt,
				int _inxr, int _inyb,
				int _outxl, int _outyt,
				int _outxr, int _outyb,
					 
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),preset(_preset),
					nphotos( _nphotos), position(_position), 
				startframe( _startframe), endframe( _endframe),
				photoframe( _photoframe), border(_border),
				inxl( _inxl), inyt( _inyt),
				inxr( _inxr), inyb( _inyb),
				outxl( _outxl), outyt( _outyt),
				outxr( _outxr), outyb( _outyb)
						
			
{
	char * Tname="Album";	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (vi.pixel_type != rvi.pixel_type)
		env->ThrowError("Album:Clips have differing pixel_types");
//	if(!vi.num_frames == rvi.num_frames)
//		env->ThrowError("Album:Clips have unequal lengths");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("Album: This color format is not supported here");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("Album: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("Album: Height/width of frame are not multiple of 4");
	
	if(startframe<0 || startframe>=vi.num_frames)
		env->ThrowError("%s:startframe number on base clip is outside its range.", Tname);
	if(endframe == -1)
		endframe = vi.num_frames - 1;
	if(endframe<0 || endframe> vi.num_frames || endframe< startframe + 8)
		env->ThrowError("%s:endframe number on base clip is incorrect.", Tname);
	if(photoframe <0)
	{
		photoframe=-photoframe;
		still=false;
	}
	else
		still=true;
	if(photoframe>=rvi.num_frames)
		env->ThrowError("%s:photoframe number on Album clip does not exist.", Tname);
	if(!still && photoframe-startframe+endframe+1 >rvi.num_frames)
		env->ThrowError("%s: Album clip is short and does not have that many frames for resizing .", Tname);

	if(preset)
	{
		if(position <1)
			env->ThrowError("%s:position must be more than 0 .", Tname);
		if (nphotos >16 || nphotos <1)
			env->ThrowError("%s:nphotos should not be more than 16 and less than 1", Tname);
		if (nphotos <5 && position > 8 )
			env->ThrowError("%s:for these nphotos position must be <9" , Tname);
		if (nphotos>4 && nphotos <10 && position >9)
			env->ThrowError("%s:for these nphotos position must be <10" , Tname);
		if (nphotos>9 && nphotos <17 && position >16)
			env->ThrowError("%s:for these nphotos position must be <17" , Tname);

		if (border > vi.height/40 || border > vi.width/40)
			env->ThrowError("%s:border is too large" , Tname);
		root=sqrt(nphotos);
		if(root*root < nphotos)
			root++;
		if(vi.height/root-2*border<4 || vi.width/root-2*border<4)
			env->ThrowError("%s:photo size for this number of clips and border width is too small", Tname);

	}
	else
	{
		if(inxr == RAND_MAX)
			inxr=vi.width-1;
		if(inyb ==RAND_MAX)
			inyb=vi.height-1;
		if(outxr == RAND_MAX)
			outxr=vi.width-1;
		if (outyb == RAND_MAX)
			outyb=vi.height-1;

		if(inxl <0 || inxl>=vi.width || inyt <0 || inyt>=vi.height
			|| inxr <0 || inxr>=vi.width || inyb <0 || inyb>=vi.height
			|| outxl <0 || outxl>=vi.width || outyt <0 || outyt>=vi.height
			|| outxr <0 || outxr>=vi.width || outyb <0 || outyb>=vi.height)
			env->ThrowError("%s:x values should be between 0 and %d, Y values should be between 0 and %d", Tname, vi.width, vi.height);
		if(inxl >= inxr || inyt >=inyb 
			|| outxl >=outxr || outyt >=outyb )
			env->ThrowError("%s:top left input or output coordinates must be less than bottom right coordinates.", Tname);
	}

}
// This is where any actual destructor code used goes
Album::~Album() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
	PVideoFrame __stdcall Album::GetFrame(int en, IScriptEnvironment* env)
	{
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		
		
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		if (en<startframe || en>endframe)
			return LeftFrame;
		int n=still?photoframe:photoframe+en-startframe;
		
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		
		int kb = vi.BitsPerPixel()/8;
		int xcoord, ycoord, width, height,inx, iny, inwidth, inheight;
		if(preset)
		{
			int root=sqrt(nphotos);
				if(root*root < nphotos)
				root++;
				xcoord=(((position-1)%root)*bwd)/root+border;
				ycoord=(((position-1)/root)*bht)/root+border;
				width=bwd/root-2*border;
				height=bht/root-2*border;

			if(nphotos>1 && nphotos <5)
			{
				
	
				if(position == 5)
				{
					xcoord=bwd/4+border;
					ycoord= border;
					width=bwd/2-2*border;
					height=bht/2-2*border;
				}
				else if(position == 6)
				{
					xcoord=bwd/4+border;
					ycoord= bht/2+border;
					width=bwd/2-2*border;
					height=bht/2-2*border;
				}
				else if(position == 7)
				{
					xcoord=border;
					ycoord= bht/4+border;
					width=bwd/2-2*border;
					height=bht/2-2*border;
				}
				else if(position == 8)
				{
					xcoord=bwd/2+border;
					ycoord= bht/4+border;
					width=bwd/2-2*border;
					height=bht/2-2*border;
				}
			}
			
			inx=0;
			iny=0;
			inwidth=bwd;
			inheight=bht;

		}
		else
		{
			xcoord=outxl;
			ycoord=outyt;
			width=outxr-outxl+1;
			height=outyb-outyt+1;
			inx=inxl;
			iny=inyt;
			inwidth= inxr-inxl+1;
			inheight=inyb-inyt+1;
		}
		if(vi.IsRGB32() || vi.IsRGB24())
		{
			ycoord=bht-(ycoord+height);
			iny=bht-(iny+inheight);
		}

		xcoord=xcoord & 0xfffffffc;
		ycoord= ycoord & 0xfffffffc;
		width = width & 0xfffffffc;
		height = height & 0xfffffffc;
		inx=inx & 0xfffffffc;
		iny=iny & 0xfffffffc;
		inwidth= inwidth & 0xfffffffc;
		inheight=inheight & 0xfffffffc;

//		env->ThrowError("x=%d, y=%d, wid= %d, ht=%d, inx=%d, iny =%d, inht=%d, inwd=%d", xcoord,ycoord,width,height,inx,iny,inheight,inwidth);

		env->MakeWritable(&LeftFrame);
		unsigned char* LeftFramep= LeftFrame->GetWritePtr();
		const unsigned char * RightFramep= RightFrame->GetReadPtr();

		const	int lpitch = LeftFrame->GetPitch();	
		const	int rpitch = RightFrame->GetPitch();

		const unsigned char * rpU, *rpV;
		unsigned char *lpU, *lpV, *wpU, *wpV;

		const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
		const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
		PVideoFrame work= env->NewVideoFrame(vi);
		unsigned char * workp=work->GetWritePtr();
		const int wpitch=work->GetPitch();
		const int wpitchUV=work->GetPitch(PLANAR_U);

		if(vi.IsYV12())
		{
			lpU = LeftFrame->GetWritePtr(PLANAR_U);
			rpU = RightFrame->GetReadPtr(PLANAR_U);
			lpV = LeftFrame->GetWritePtr(PLANAR_V);
			rpV = RightFrame->GetReadPtr(PLANAR_V);
			wpV = work->GetWritePtr(PLANAR_V);
			wpU = work->GetWritePtr(PLANAR_U);
		}
				

			
		if(vi.IsRGB32())
		{
			HFastResizerRGB32(RightFramep+(iny)*rpitch+4*inx, workp, inheight, inwidth,
								inheight, width, rpitch, wpitch, false);
			VFastResizerRGB32(workp, LeftFramep+lpitch*(ycoord)+4*xcoord,
								inheight, width, height, width, wpitch,lpitch, false);
		}

		if(vi.IsRGB24())
		{
			HFastResizerRGB24(RightFramep+(iny)*rpitch+3*inx, workp, inheight, inwidth,
								inheight, width, rpitch, wpitch, false);
			VFastResizerRGB24(workp, LeftFramep+lpitch*(ycoord)+3*xcoord,
								inheight, width, height, width, wpitch,lpitch, false);
		}
		if(vi.IsYUY2())
		{
			HFastResizerYUY2(RightFramep+iny*rpitch+2*inx, workp, inheight, inwidth,
								inheight, width, rpitch, wpitch, false);
			VFastResizerYUY2(workp, LeftFramep+lpitch*ycoord+2*xcoord,
								inheight, width, height, width, wpitch,lpitch, false);
		}
		if(vi.IsYV12())
		{
			HFastResizerYV12(RightFramep+iny*rpitch+inx, workp, inheight, inwidth,
								inheight, width, rpitch, wpitch, false);
			VFastResizerYV12(workp, LeftFramep+lpitch*ycoord+xcoord,
								inheight, width, height, width, wpitch,lpitch, false);
			HFastResizerYV12(rpU+iny/2*rpitchUV+inx/2, wpU, inheight/2, inwidth/2,
								inheight/2, width/2, rpitchUV, wpitchUV, false);
			VFastResizerYV12(wpU, lpU+lpitchUV*ycoord/2+xcoord/2,
								inheight/2, width/2, height/2, width/2, wpitchUV,lpitchUV, false);
			HFastResizerYV12(rpV+iny/2*rpitchUV+inx/2, wpV, inheight/2, inwidth/2,
								inheight/2, width/2, rpitchUV, wpitchUV, false);
			VFastResizerYV12(wpV, lpV+lpitchUV*ycoord/2+xcoord/2,
								inheight/2, width/2, height/2, width/2, wpitchUV,lpitchUV, false);


		}



		return LeftFrame;
		
	}

/********** 
bool HFastResizerRGB32(const unsigned char* sptr,unsigned char* dptr,int sht, int swd,int dht, int dwd,
					 int spitch, int dpitch,  bool hflip=false);

bool VFastResizerRGB32(const unsigned char* srcptr,unsigned char* dstptr,int srcht, int srcwd,int dstht, int dstwd,
					 int spitch, int dpitch,  bool vflip=false);


/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_Album(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	
				
	return new Album(args[0].AsClip(),	// clip as Base Clip
							args[1].AsClip(),	// Clip as Album Clip   
 							args[2].AsBool(true),	//preset
							args[3].AsInt(9),	// Number of photos in Album page
							args[4].AsInt(1),	// position of  photo on Album page 
							args[5].AsInt(0),	// start frame of base clip	
							args[6].AsInt(-1),	// end frame of base clip
							args[7].AsInt(0),	// Photo frame number on album clip. If -ve resized motion.
							args[8].AsInt(4),	// border width
							args[9].AsInt(0),	// inxl input left x value
							args[10].AsInt(0),	// input top  y coord
							args[11].AsInt(RAND_MAX), //input right x
							args[12].AsInt(RAND_MAX),	// input bottom  y
							args[13].AsInt(args[9].AsInt(0)), //output left x
							args[14].AsInt(args[10].AsInt(0)), // output top y
							args[15].AsInt(args[11].AsInt(RAND_MAX)), // ouput  right x
							args[16].AsInt(args[12].AsInt(RAND_MAX)), // output bottom y
							env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("Album", "cc[preset]b[nphotos]i[position]i[startframe]i[endframe]i[photoframe]i[border]i[inxl]i[inyt]i[inxr]i[inyb]i[outxl]i[outyt]i[outxr]i[outyb]i", Create_Album, 0);
				
    return "Album  Plugin";
}
*/
