/******************************************************************************
AlbumPop filter plugin for Avisynth by V.C.Mohan
On a base clip  resized image from of right clip is overlaid. The position of clips, 
preset or custom position at which to display are provided. The start frame, end frame,
 corresponding RightClip frame number and coordinates are parameters. If preset is true,
 x and y coordinates are disregarded. If preset is false, nphotos and position, border
 are disregarded. nphotos number does not include base clip. 
********************************************************************************/
/*  
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
*/
class AlbumPop : public GenericVideoFilter {
		PClip RightClip;	// AlbumPop clip
		
		int nphotos;		// number of photos in this AlbumPop page
		int position;		// RightClip position
		
		int PopUpFrame;		//  frame number at which Photo pops to full size motion
		int CutoffFrame;	// frame number at which Photo Clip closes out
		bool pass1;
		int photoframe;		// First Frame number for photo. rest for video 
		int border;			// border pixels. not more than (bwd(and bht)/40
		
		__int64 audio_base_end ;
		__int64 audio_base_start;
		__int64 audio_mont_start;
		
		char *dbuf, *Ubuf, *Vbuf;
		 
  public:
							//Definition of function
    AlbumPop(PClip _child, PClip _RightClip,
				int _nphotos, int _position, 
				
				int _PopUpFrame,
				int _CutoffFrame,
				bool _pass1,int _photoframe,
				  
				IScriptEnvironment* env) ;	
			
 	~AlbumPop();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
bool __stdcall GetParity(int n);

};	

//Here is the acutal constructor code used

AlbumPop::AlbumPop(PClip _child, PClip _RightClip,
					 int _nphotos, int _position, 
				
				int _PopUpFrame,int _CutoffFrame,
				bool _pass1,int _photoframe,
					 
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
					nphotos( _nphotos), position(_position), 
				
				PopUpFrame( _PopUpFrame),CutoffFrame(_CutoffFrame),
				pass1(_pass1),photoframe(_photoframe)
				
			
{
	char * Tname="AlbumPop";	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (vi.pixel_type != rvi.pixel_type)
		env->ThrowError("AlbumPop:Clips have differing pixel_types");
//	if(!vi.num_frames == rvi.num_frames)
//		env->ThrowError("AlbumPop:Clips have unequal lengths");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("AlbumPop: This color format is not supported here");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("AlbumPop: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("AlbumPop: Height/width of frame are not multiple of 4");
	border=(vi.width<vi.height?vi.width/80:vi.height/80) & 0xfffffffc;
	if(photoframe <0 || photoframe>=rvi.num_frames)
		env->ThrowError("%s: AlbumPop clip does not have thumbnail frame number.", Tname);


	if(PopUpFrame<0 || PopUpFrame>=vi.num_frames || CutoffFrame<0 || CutoffFrame>vi.num_frames)
		env->ThrowError("%s: base clip does not have the frame numbers.", Tname);
	if(CutoffFrame-PopUpFrame+1>rvi.num_frames)
		env->ThrowError("%s: AlbumPop clip does not have that many frame numbers.", Tname);




	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
//		if(vi.sample_type & SAMPLE_INT24)
//			env->ThrowError("%s: 24 bit audio format is not supported. Please convert to a decent format",Tname);


	}


	audio_base_end = vi.AudioSamplesFromFrames(CutoffFrame+1)-1;
	audio_base_start = vi.AudioSamplesFromFrames(PopUpFrame);
	audio_mont_start=vi.AudioSamplesFromFrames(photoframe);

}
// This is where any actual destructor code used goes
AlbumPop::~AlbumPop() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
	

PVideoFrame __stdcall AlbumPop::GetFrame(int en, IScriptEnvironment* env)
	{
		PVideoFrame LeftFrame = child->GetFrame(en, env);

		PVideoFrame RightFrame;
		
			
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();

		const	int bwd = lvi.width;
		const	int bht = LeftFrame->GetHeight();
		

		int kb = vi.BitsPerPixel()/8;
		int xcoord, ycoord, width, height,inx=0, iny=0, inwidth=bwd, inheight=bht;
		
		if(pass1)
		{
			if(en==0)
			{
						// Compute thumb nail dimensions
				int root=sqrt(nphotos);
				if(root*root < nphotos)
					root++;
				xcoord=(((position-1)%root)*bwd)/root+border;
				ycoord=(((position-1)/root)*bht)/root+border;
				width=bwd/root-2*border;
				height=bht/root-2*border;

			}

		}
		if(!pass1)
		{
			if(en>=PopUpFrame && en < PopUpFrame+8)
			{
					// compute expanding frame dimensions
				int root=sqrt(nphotos);
				if(root*root < nphotos)
					root++;
				int x=(((position-1)%root)*bwd)/root+border;
				int y=(((position-1)/root)*bht)/root+border;
			
				xcoord=x-x*(en-PopUpFrame)/8;
				ycoord=y-y*(en-PopUpFrame)/8;
				int w=bwd/root-2*border;
				int h=bht/root-2*border;
				width=w+((bwd-w)*(en-PopUpFrame))/8;
				height=h+((bht-h)*(en-PopUpFrame))/8;
				

			}
			else
			{
					// pass the frame as it is
				if (en>=PopUpFrame+8 && en<=CutoffFrame)
					return  RightClip->GetFrame(en-PopUpFrame+photoframe, env);
					
				return LeftFrame;
			}
		}

		env->MakeWritable(&LeftFrame);
		const	int lpitch = LeftFrame->GetPitch();	
		unsigned char* LeftFramep= LeftFrame->GetWritePtr();
		const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
		const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
		const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
		unsigned char *lpU = LeftFrame->GetWritePtr(PLANAR_U);				
		unsigned char *lpV = LeftFrame->GetWritePtr(PLANAR_V);
//	env->ThrowError("x=%d, y=%d, wid= %d, ht=%d, inx=%d, iny =%d, inht=%d, inwd=%d", xcoord,ycoord,width,height,inx,iny,inheight,inwidth);
	
		if((en ==0 && pass1) || !pass1)
		{
					// Resize either to photos or as expanding from photo to full frame

			if(vi.IsRGB24()|| vi.IsRGB32())
			{
				ycoord=bht-(ycoord+height);
				iny=bht-(iny+inheight);
			}
			xcoord=xcoord & 0xfffffffc;
			ycoord= ycoord & 0xfffffffc;
			width = width & 0xfffffffc;
			height = height & 0xfffffffc;
			inx=inx & 0xfffffffc;
			iny=iny & 0xfffffffc;
			inwidth= inwidth & 0xfffffffc;
			inheight=inheight & 0xfffffffc;

//		env->ThrowError("x=%d, y=%d, wid= %d, ht=%d, inx=%d, iny =%d, inht=%d, inwd=%d", xcoord,ycoord,width,height,inx,iny,inheight,inwidth);
			if (pass1)
				RightFrame = RightClip->GetFrame(photoframe, env);
			if (!pass1)
				RightFrame = RightClip->GetFrame(en-PopUpFrame+photoframe, env);

			const int rpitch = RightFrame->GetPitch();
			const unsigned char * RightFramep= RightFrame->GetReadPtr();
			const unsigned char * rpU, *rpV;
			unsigned char  *wpU, *wpV;
			const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
			PVideoFrame work= env->NewVideoFrame(vi);
			unsigned char * workp=work->GetWritePtr();
			const int wpitch=work->GetPitch();
			const int wpitchUV=work->GetPitch(PLANAR_U);

//	env->ThrowError("x=%d, y=%d, wid= %d, ht=%d, inx=%d, iny =%d, inht=%d, inwd=%d", xcoord,ycoord,width,height,inx,iny,inheight,inwidth);
				
			if(vi.IsRGB32())
			{
				HFastResizerRGB32(RightFramep+(iny)*rpitch+4*inx, workp, inheight, inwidth,
									inheight, width, rpitch, wpitch, false);
				VFastResizerRGB32(workp, LeftFramep+lpitch*(ycoord)+4*xcoord,
									inheight, width, height, width, wpitch,lpitch, false);
			}

			if(vi.IsRGB24())
			{
				HFastResizerRGB24(RightFramep+(iny)*rpitch+3*inx, workp, inheight, inwidth,
									inheight, width, rpitch, wpitch, false);
				VFastResizerRGB24(workp, LeftFramep+lpitch*(ycoord)+3*xcoord,
									inheight, width, height, width, wpitch,lpitch, false     );
			}
//	env->ThrowError("x=%d, y=%d, wid= %d, ht=%d, inx=%d, iny =%d, inht=%d, inwd=%d", xcoord,ycoord,width,height,inx,iny,inheight,inwidth);

			if(vi.IsYUY2())
			{
				HFastResizerYUY2(RightFramep+iny*rpitch+2*inx, workp, inheight, inwidth,
									inheight, width, rpitch, wpitch, false);
				VFastResizerYUY2(workp, LeftFramep+lpitch*ycoord+2*xcoord,
									inheight, width, height, width, wpitch,lpitch, false);
			}
			if(vi.IsYV12())
			{
				
				rpU = RightFrame->GetReadPtr(PLANAR_U);
				rpV = RightFrame->GetReadPtr(PLANAR_V);
				wpV = work->GetWritePtr(PLANAR_V);
				wpU = work->GetWritePtr(PLANAR_U);

				HFastResizerYV12(RightFramep+iny*rpitch+inx, workp, inheight, inwidth,
									inheight, width, rpitch, wpitch, false);
				VFastResizerYV12(workp, LeftFramep+lpitch*ycoord+xcoord,
									inheight, width, height, width, wpitch,lpitch, false);
				HFastResizerYV12(rpU+iny/2*rpitchUV+inx/2, wpU, inheight/2, inwidth/2,
									inheight/2, width/2, rpitchUV, wpitchUV, false);
				VFastResizerYV12(wpU, lpU+lpitchUV*ycoord/2+xcoord/2,
									inheight/2, width/2, height/2, width/2, wpitchUV,lpitchUV, false);
				HFastResizerYV12(rpV+iny/2*rpitchUV+inx/2, wpV, inheight/2, inwidth/2,
									inheight/2, width/2, rpitchUV, wpitchUV, false);
				VFastResizerYV12(wpV, lpV+lpitchUV*ycoord/2+xcoord/2,
									inheight/2, width/2, height/2, width/2, wpitchUV,lpitchUV, false);


			}
		}

//	env->ThrowError("en= %d, pos=%d, photos=%d",en, position,nphotos);
		
		if(en==0 && position==nphotos && pass1)
		{	
			dbuf=new char[bht*lpitch];
		//	BitBlt(BYTE* dstp, int dst_pitch, const BYTE* srcp, int src_pitch, int row_size, int height) = 0;
		//	BitBlt(buf,pitch,LeftFramep,pitch,kb*bwd,bht);
			for(int h=0; h<bht; h++)
				for(int w=0; w<bwd; w++)
					for(int k=0;k<kb; k++)

						*(dbuf+h*lpitch+kb*w+k)=*(LeftFramep+h*lpitch+kb*w+k);
			
//	env->ThrowError("en= %d, pos=%d, photos=%d",en, position,nphotos);
			if(vi.IsYV12())
			{
				
				Ubuf= new char[bhtUV*lpitchUV];
				Vbuf= new char[bhtUV*lpitchUV];
			//	BitBlt(Ubuf,pitchUV,lpU,pitchUV,bwdUV,bhtUV);
			//	BitBlt(Vbuf,pitchUV,lpV,pitchUV,bwdUV,bhtUV);
				for(int h=0; h<bhtUV; h++)
					for(int w=0; w<bwdUV; w++)
					{
						*(Ubuf+h*lpitchUV+w)=*(lpU+h*lpitchUV+w);
						*(Vbuf+h*lpitchUV+w)=*(lpV+h*lpitchUV+w);
					}

			}
		}
		else if(en >0 && position ==nphotos && pass1)
		{
		

			for(int h=0; h<bht; h++)
				for(int w=0; w<bwd; w++)
					for(int k=0;k<kb; k++)

						*(LeftFramep+h*lpitch+kb*w+k)=*(dbuf+h*lpitch+kb*w+k);
//	env->ThrowError("en= %d, pos=%d, photos=%d",en, position,nphotos);
			if(vi.IsYV12())
			{
				
//				BitBlt(lpU,pitchUV,Ubuf,pitchUV,bwdUV,bhtUV);
//				BitBlt(lpV,pitchUV,Vbuf,pitchUV,bwdUV,bhtUV);
				for(int h=0; h<bhtUV; h++)
					for(int w=0; w<bwdUV; w++)
					{
						*(lpU+h*lpitchUV+w)=*(Ubuf+h*lpitchUV+w);
						*(lpV+h*lpitchUV+w)=*(Vbuf+h*lpitchUV+w);
					}

			}
			if(en==vi.num_frames-1)
			{
				delete []dbuf;
				delete []Ubuf;
				delete []Vbuf;
			}
			
		}

		return LeftFrame;
		
	}

/********** 
bool HFastResizerRGB32(const unsigned char* sptr,unsigned char* dptr,int sht, int swd,int dht, int dwd,
					 int spitch, int dpitch,  bool hflip=false);

bool VFastResizerRGB32(const unsigned char* srcptr,unsigned char* dstptr,int srcht, int srcwd,int dstht, int dstwd,
					 int spitch, int dpitch,  bool vflip=false);


/***************************************************************/

void AlbumPop::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
	if(pass1)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
  
 
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count < audio_base_start || start > audio_base_end )
	{
		child->GetAudio(buf, start, count, env);
		return;
	}

	const __int64 bytes = vi.BytesFromAudioSamples(count);
 
	if(start<audio_base_start ) // case 
	{
		void* tabs=(char *)buf+(audio_base_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_base_end)
		{
			
			__int64 tcount=count-(audio_base_start-start);	// right clip samples
			child->GetAudio(buf, start,count-tcount, env);
			RightClip->GetAudio(tabs, audio_mont_start, tcount, env);

		}
		if(start+count>=audio_base_end)
		{
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(tabs,
				audio_mont_start, audio_base_end-audio_base_start+1, env);
			
		}
	}
	if(start>audio_base_start )
	{
		
		if(start+count<audio_base_end)
		{
		
			
			RightClip->GetAudio(buf, start-audio_base_start+audio_mont_start, count, env);
		
		}
		if(start+count>=audio_base_end)
		{

		
			__int64 tcount=count-(audio_base_end-start+1);
			void* tabs=(char *)buf+(audio_base_end-start+1)*vi.BytesPerAudioSample();
			child->GetAudio(tabs, audio_base_end+1,tcount, env);
			RightClip->GetAudio(buf, start-audio_base_start+audio_mont_start+1,audio_base_end-start+1, env);
		
		}
	}

}
bool AlbumPop::GetParity(int n) 
{
	if(pass1)
		return child->GetParity(n);


	return (n < PopUpFrame || n>CutoffFrame) ? child->GetParity(n) : RightClip->GetParity(0+n-PopUpFrame);
}








// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_AlbumPop(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	if(args[1].AsBool() && 3*args[3].ArraySize() != args[4].ArraySize())
		env->ThrowError("AlbumPop: There are %d entries in frame number array instead of %d",args[4].ArraySize(),3*args[3].ArraySize());
	if(!args[1].AsBool() && 2*args[3].ArraySize() != args[4].ArraySize())
		env->ThrowError("AlbumPop: There are %d entries in frame number array instead of %d",args[4].ArraySize(),2*args[3].ArraySize());
		
//	if(args[4][0].AsInt()<60)
//		env->ThrowError("AlbumPop:The first expand frame number must be atleast 60");

	for(int i=0;i<args[3].ArraySize();i++)
	{
		if(args[1].AsBool())
			if(args[4][3*i+1].AsInt() <= args[4][3*i].AsInt()+8)
				env->ThrowError("AlbumPop:cutoff frame number %d must be atleast 8 more than expand frame given as %d",args[4][3*i+1].AsInt() , args[4][3*i].AsInt()); 
		if(!args[1].AsBool())
			if(args[4][2*i+1].AsInt() <= args[4][2*i].AsInt()+8)
				env->ThrowError("AlbumPop:cutoff frame number %d must be atleast 8 more than expand frame given as %d",args[4][2*i+1].AsInt() , args[4][2*i].AsInt()); 

	}

	
	for(i=0;i<args[3].ArraySize()-1;i++)
	{
		if(!args[1].AsBool())
			if(args[4][2*i+1].AsInt() >= args[4][2*(i+1)].AsInt())
				env->ThrowError("AlbumPop:cutoff frame number %d must be less than next expand frame given as %d",args[4][2*i+1].AsInt() , args[1][2*i+2].AsInt()); 
		if(args[1].AsBool())
			if(args[4][3*i+1].AsInt() >= args[4][3*(i+1)].AsInt())
				env->ThrowError("AlbumPop:cutoff frame number %d must be less than next expand frame given as %d",args[4][3*i+1].AsInt() , args[1][3*i+3].AsInt()); 

	}
	int x=args[1].AsBool()?3:2;

	PClip result=args[0].AsClip();
	if(args[2].AsInt()<2)
		for(i=0;i<args[3].ArraySize();i++)
		{

			result= new AlbumPop(result,			// LeftClip as Base Clip
					args[3][i].AsClip(),	// RightClip as AlbumPop Clip  
 					args[3].ArraySize(),	// number of photos in album page
					 
					i+1,				// position number of photo
					args[4][x*i].AsInt(4),	// Frame number at which to pop		
					args[4][x*i+1].AsInt(1),	// frame number on which to cut off
					true,						// is pass1 
					args[1].AsBool()?args[4][3*i+2].AsInt():0, // photo obtained from frame number

							
							env);
		}
	if(args[2].AsInt() !=1)
		for(i=0;i<args[3].ArraySize();i++)
		{

			result= new AlbumPop(result,	// LeftClip as Base Clip
					args[3][i].AsClip(),	// RightClip as AlbumPop Clip  
 					args[3].ArraySize(),	// number of photos in album page
					 
					i+1,				// position number of photo
					args[4][x*i].AsInt(4),	// Frame number at which to popup		
					args[4][x*i+1].AsInt(1),	// frame number on which to cut off
					false,						// is pass1? No this is pass2
					args[1].AsBool()?args[4][3*i+2].AsInt():0,	// 
							
							env);
		}
	return result;

// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("AlbumPop", "cbic+i+", Create_AlbumPop, 0);
				
    return "AlbumPop  Plugin";
}
*/
