/******************************************************************************
PopUp filter plugin for Avisynth by V.C.Mohan
Using a thumbnailed clip as base clip, 
********************************************************************************/
/*  
#include "windows.h"
#include "avisynth.h"
#include "FastResizer.cpp"
*/
class PopUp : public GenericVideoFilter {
		PClip RightClip;	// Montage clip
	//	int nclips;		// number of clips in this montage
	//	int position;		// montage clip position
		int PopAtFrame;		// Base clip frame number at which pops
		int endframe;		// Base clip frame number at which pop ends
		int PopInFrame;		// Frame number of RightClip which pops at PopAtFrame
		int zframes;

	//	int border;
	//	void * abuf;

		__int64 video_fade_start;
		__int64 video_fade_end;

		__int64 audio_base_end ;
		__int64 audio_base_start;
		__int64 audio_mont_start;
//		__int64 abufsize;
		
		
		 
  public:
							//Definition of function
    PopUp(PClip _child, PClip _RightClip,
			//	int _nclips, int _position, 
				int _PopAtFrame, int _endframe,
				int _PopInFrame,int _zframes,
				  
				IScriptEnvironment* env) ;	
			
 	~PopUp();				//destructor

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);					
// This is the function that AviSynth calls to get a given frame.
// So when this functions gets called, the filter is supposed to return frame n.
void __stdcall GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env);
bool __stdcall GetParity(int n);

};	
/*************************************************
 * The following is the implementation 
 * of the defined functions.
 *************************************************/
void PopUp::GetAudio(void* buf, __int64 start, __int64 count, IScriptEnvironment* env) 
{
  
 
	const VideoInfo& rvi= RightClip->GetVideoInfo();

	if (start+count <= audio_base_start)
	{
		child->GetAudio(buf, start, count, env);
		return;
	}
	else if (start > audio_base_end)
	{
		RightClip->GetAudio(buf, start -audio_base_start, count, env);
		return;
	}
	const __int64 bytes = vi.BytesFromAudioSamples(count);
 
	if(start<audio_base_start ) // case 
	{
		void* tabs=(char *)buf+(audio_base_start-start)*vi.BytesPerAudioSample(); // audio buffer start for transition
		if(start+count<audio_base_end)
		{
			
			__int64 tcount=count-(audio_base_start-start);	// right clip samples
			child->GetAudio(buf, start,count-tcount, env);
			RightClip->GetAudio(tabs, audio_mont_start, tcount, env);

		}
		if(start+count>=audio_base_end)
		{
			child->GetAudio(buf, start, count, env);
			RightClip->GetAudio(tabs,
				audio_mont_start, audio_base_end-audio_base_start+1, env);
			
		}
	}
	if(start>audio_base_start )
	{
		
		if(start+count<audio_base_end)
		{
		
			
			RightClip->GetAudio(buf, start-audio_base_start+audio_mont_start, count, env);
		
		}
		if(start+count>=audio_base_end)
		{

			__int64 tcount=count-(audio_base_end-start+1);
			void* tabs=(char *)buf+(audio_base_end-start+1)*vi.BytesPerAudioSample();
			child->GetAudio(tabs, audio_base_end+1,tcount, env);
			RightClip->GetAudio(buf, start-audio_base_start+audio_mont_start+1,audio_base_end-start+1, env);
			
		}
	}

}
bool PopUp::GetParity(int n) 
{
  return (n < PopAtFrame || n>endframe) ? child->GetParity(n) : RightClip->GetParity(PopInFrame+n-PopAtFrame);
}


//Here is the acutal constructor code used

PopUp::PopUp(PClip _child, PClip _RightClip, 
				int _PopAtFrame, int _endframe,
				int _PopInFrame,int _zframes,
					 
					 IScriptEnvironment* env) :
	GenericVideoFilter(_child), RightClip(_RightClip),
 
				PopAtFrame( _PopAtFrame), endframe( _endframe),
				PopInFrame( _PopInFrame),zframes( _zframes)
						
			
{
	char * Tname="PopUp";	
	const VideoInfo& rvi= RightClip->GetVideoInfo();
	if (!vi.pixel_type == rvi.pixel_type)
		env->ThrowError("PopUp:Clips have differing pixel_types");
	if(!vi.num_frames == rvi.num_frames)
		env->ThrowError("PopUp:Clips have unequal lengths");
	if(!vi.IsRGB24() && !vi.IsRGB32() && !vi.IsYUY2() && !vi.IsYV12())
		env->ThrowError("PopUp: This color format is not supported here");
	if(!(vi.height==rvi.height) || !(vi.width == rvi.width))
		env->ThrowError("PopUp: The heights/widths of clips are unequal");
	if(!(vi.height & 3)==0 || !(vi.width & 3)==0)
		env->ThrowError("PopUp: Height/width of frame are not multiple of 4");

	if(PopAtFrame<0 || PopAtFrame>=vi.num_frames)
		env->ThrowError("%s:PopAtFrame number on base clip is outside its range.", Tname);
	if(endframe<0 || endframe> vi.num_frames || endframe< PopAtFrame + 2* zframes)
		env->ThrowError("%s:endframe number on base clip is incorrect.", Tname);
	if(PopInFrame<0 || PopInFrame+endframe-PopAtFrame+1>=rvi.num_frames)
		env->ThrowError("%s:PopInFrame number on  clip is incorrect or base clip frame numbers are incorrect.", Tname);
	if(2*zframes>=endframe-PopAtFrame)
		env->ThrowError("%s:twice zframes must be less than endframe-PopAtFrame", Tname);
 

	if (vi.HasAudio() && rvi.HasAudio())
	{
		
		if (vi.AudioChannels() != rvi.AudioChannels())
			env->ThrowError("%s: The number of audio channels in clips are not same",Tname);

		if (vi.SamplesPerSecond() != rvi.SamplesPerSecond())
			env->ThrowError("%s: The audio of the two clips have different samplerates! Use SSRC()/ResampleAudio()",Tname);

		if (vi.sample_type != rvi.sample_type)
			env->ThrowError("%s: The audio samples of clips are in different formats.",Tname);
		if(vi.sample_type & SAMPLE_INT24)
			env->ThrowError("%s: 24 bit audio format is not acceptable",Tname);


	}


	audio_base_end = vi.AudioSamplesFromFrames(endframe+1)-1;
	audio_base_start = vi.AudioSamplesFromFrames(PopAtFrame);
	audio_mont_start= vi.AudioSamplesFromFrames(PopInFrame);
}
// This is where any actual destructor code used goes
PopUp::~PopUp() {
	
// This is where you can deallocate any memory you might have used.
}
	
/***************************************************************/	
	PVideoFrame __stdcall PopUp::GetFrame(int en, IScriptEnvironment* env)
	{
		PVideoFrame LeftFrame = child->GetFrame(en, env);
		const VideoInfo& lvi = child->GetVideoInfo();
		const VideoInfo& rvi= RightClip->GetVideoInfo();
		if(en< PopAtFrame || en >endframe)
			return LeftFrame;		 
		const	int bwd = lvi.width;
        const	int bht = LeftFrame->GetHeight();
		

		int kb=vi.BitsPerPixel()/8;
		int n=en-PopAtFrame+PopInFrame;
		PVideoFrame RightFrame = RightClip->GetFrame(n, env);
		
		int xcoord, ycoord, width, height;


		if((en>=PopAtFrame && en < PopAtFrame+zframes)
			|| (en<=endframe && en > endframe-zframes))
		{
			env->MakeWritable(&LeftFrame);
			unsigned char* LeftFramep= LeftFrame->GetWritePtr();
			const unsigned char * RightFramep= RightFrame->GetReadPtr();

			const	int lpitch = LeftFrame->GetPitch();	
			const	int rpitch = RightFrame->GetPitch();
			const unsigned char * rpU, *rpV;
			unsigned char *lpU, *lpV, *wpU, *wpV;
			const int bwdUV=LeftFrame->GetRowSize(PLANAR_U);
			const int bhtUV=LeftFrame->GetHeight(PLANAR_U);
			const int lpitchUV=LeftFrame->GetPitch(PLANAR_U);
			const int rpitchUV=RightFrame->GetPitch(PLANAR_U);
			PVideoFrame work= env->NewVideoFrame(vi);
			unsigned char * workp=work->GetWritePtr();
			const int wpitch=work->GetPitch();
			const int wpitchUV=work->GetPitch(PLANAR_U);

			if(vi.IsYV12())
			{
				lpU = LeftFrame->GetWritePtr(PLANAR_U);
				rpU = RightFrame->GetReadPtr(PLANAR_U);
				lpV = LeftFrame->GetWritePtr(PLANAR_V);
				rpV = RightFrame->GetReadPtr(PLANAR_V);
				wpV = work->GetWritePtr(PLANAR_V);
				wpU = work->GetWritePtr(PLANAR_U);
			}
				

  
			int ht,wd,x=0,y=0;
			if(en < PopAtFrame+zframes)
			{
				ht=((bht*(en-PopAtFrame))/zframes) & 0xfffffffc;
				wd=((bwd*(en-PopAtFrame))/zframes) & 0xfffffffc;
				x=(bwd-wd)/2;
				y=(bht-ht)/2;
			}
			if(en>endframe-zframes)
			{
				ht=((bht*(endframe-en))/zframes) & 0xfffffffc;
				wd=((bwd*(endframe-en))/zframes) & 0xfffffffc;
				x=(bwd-wd)/2;
				y=(bht-ht)/2;
			}
			if (ht<4 || wd<4)
				return LeftFrame;

			if(vi.IsRGB32())
			{
			
				HFastResizerRGB32(RightFramep,workp,bht,bwd,
									bht,wd,rpitch,wpitch,false);
				VFastResizerRGB32(workp,LeftFramep+(y)*lpitch+kb*x,bht,wd,ht,
									wd,wpitch,lpitch,false);
			
			}
			if(vi.IsRGB24())
			{
			
				HFastResizerRGB24(RightFramep,workp,bht,bwd,
								bht,wd,rpitch,wpitch,false);
				VFastResizerRGB24(workp,LeftFramep+(y)*lpitch+kb*x,bht,wd,ht,
								wd,wpitch,lpitch,false);
			
			}
			if(vi.IsYUY2())
			{
		
				HFastResizerYUY2(RightFramep,workp,bht,bwd,
								bht,wd,rpitch,wpitch,false);
				VFastResizerYUY2(workp,LeftFramep+y*lpitch+kb*x,bht,wd,ht,
								wd,wpitch,lpitch,false);
				
			}
			if(vi.IsYV12())
			{
			
				HFastResizerYV12(RightFramep,workp,bht,bwd,
								bht,wd,rpitch,wpitch,false);
				VFastResizerYV12(workp,LeftFramep+y*lpitch+x,bht,wd,ht,
								wd,wpitch,lpitch,false);
				HFastResizerYV12(rpU,wpU,bhtUV,bwdUV,
								bhtUV,wd/2,rpitchUV,wpitchUV,false);
				VFastResizerYV12(wpU,lpU+y/2*lpitchUV+x/2,bhtUV,wd/2,ht/2,
								wd/2,wpitchUV,lpitchUV,false);
				HFastResizerYV12(rpV,wpV,bhtUV,bwdUV,
								bhtUV,wd/2,rpitchUV,wpitchUV,false);
				VFastResizerYV12(wpV,lpV+y/2*lpitchUV+x/2,bhtUV,wd/2,ht/2,
								wd/2,wpitchUV,lpitchUV,false);
				
			}
			
			return LeftFrame;

			
		}
		return RightFrame;
	}


/********** 



/***************************************************************/
// This is the function that created the filter, when the filter has been called.
// This can be used for simple parameter checking, so it is possible to create different filters,

AVSValue __cdecl Create_PopUp(AVSValue args, void* user_data, IScriptEnvironment* env) 
{
	
				
	return new PopUp(args[0].AsClip(),	// clip as Base(Montaged clip) Clip
						args[1].AsClip(),	// Clip as Video  Clip   
						args[2].AsInt(),	// frame of base clip in which to zoom in
						args[3].AsInt(),	// frame of base clip in which to zoom out
						args[4].AsInt(),	// frame number of video clip corresponding to
											// Zoom in frame of base clip
						args[5].AsInt(8),	// Number of frames to zoom in or out
						env);
// Calls the constructor with the arguments provied.
}
// The following function is the function that actually registers the filter in AviSynth
// It is called automatically, when the plugin is loaded to see which functions this filter contains.
/*
extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{
    env->AddFunction("PopUp", "cciii[zframes]i", Create_PopUp, 0);
				
    return "PopUp  Plugin";
}
*/
